#ifndef _INC_COLOR_
#define _INC_COLOR_

class Color32;

class Color {
	float red;
	float green;
	float blue;
	float alpha;
public:
	Color();
	Color(Color32 col);
	Color(float red, float green, float blue, float alpha);
	Color(float red, float green, float blue);
	void setColor(float red, float green, float blue, float alpha);
	void setColor(float red, float green, float blue);
	void setRed(float red);
	void setGreen(float green);
	void setBlue(float blue);
	void setAlpha(float alpha);
	float getRed() const;
	float getGreen() const;
	float getBlue() const;
	float getAlpha() const;
	unsigned char getRed8() const;
	unsigned char getGreen8() const;
	unsigned char getBlue8() const;
	unsigned char getAlpha8() const;
	void clampTo01();
	void setBlack();
	void setWhite();
	void setGrey(float shade);
	Color lerp(const Color& col2, float factor) const;
	Color operator+(const Color& col2) const;
	Color operator-(const Color& col2) const;
	Color operator*(const Color& col2) const;
	Color operator*(float factor) const;
	Color operator/(float factor) const;
	bool operator==(const Color& col2) const;
	bool operator!=(const Color& col2) const;
	Color operator+=(const Color& col2);
	Color operator-=(const Color& col2);
	Color operator*=(const Color& col2);
	Color operator*=(float factor);
	Color operator/=(float factor);
	Color operator+(void) const;
	Color operator-(void) const;
	friend Color operator*(float factor, const Color& col);
};

class Color32 {
protected:
	unsigned char comp[4];
public:
	Color32();
	Color32(Color col);
	Color32(unsigned char red, unsigned char green, unsigned char blue, unsigned char alpha);
	Color32(unsigned char red, unsigned char green, unsigned char blue);
	Color32 operator=(Color col);
	unsigned char& operator()(int index);
	unsigned char& refComp(int index);
	unsigned char* ptrComp(int index);
	friend int color32SSD(const Color32& col1, const Color32& col2);
	friend Color32 color32lerp(const Color32& col1, const Color32& col2, float factor);
	Color32 operator+(const Color32& col2) const;
	Color32 operator*(float factor) const;
	friend Color32 operator*(float factor, const Color32& col);
};

inline Color::Color() : red(0.0f), green(0.0f), blue(0.0f), alpha(1.0f) { }

inline Color::Color(Color32 col) {
	red = col(0) / 256.0f;
	green = col(1) / 256.0f;
	blue = col(2) / 256.0f;
	alpha = col(3) / 256.0f;
}

inline Color::Color(float red, float green, float blue, float alpha) {
	this->red = red;
	this->green = green;
	this->blue = blue;
	this->alpha = alpha;
}

inline Color::Color(float red, float green, float blue) {
	this->red = red;
	this->green = green;
	this->blue = blue;
	this->alpha = 1.0f;
}

inline void Color::setColor(float red, float green, float blue, float alpha) {
	this->red = red;
	this->green = green;
	this->blue = blue;
	this->alpha = alpha;
}

inline void Color::setColor(float red, float green, float blue) {
	this->red = red;
	this->green = green;
	this->blue = blue;
	this->alpha = 1.0f;
}

inline void Color::setRed(float red) { this->red = red; }

inline void Color::setGreen(float green) { this->green = green; }

inline void Color::setBlue(float blue) { this->blue = blue; }

inline void Color::setAlpha(float alpha) { this->alpha = alpha; }

inline float Color::getRed() const { return red; }

inline float Color::getGreen() const { return green; }

inline float Color::getBlue() const { return blue; }

inline float Color::getAlpha() const { return alpha; }

inline unsigned char Color::getRed8() const {
	if(red > 0.999f) return 255;
	if(red < 0.0f) return 0;
	return (unsigned char) (red * 256.0f);
}

inline unsigned char Color::getGreen8() const {
	if(green > 0.999f) return 255;
	if(green < 0.0f) return 0;
	return (unsigned char) (green * 256.0f);
}

inline unsigned char Color::getBlue8() const {
	if(blue > 0.999f) return 255;
	if(blue < 0.0f) return 0;
	return (unsigned char) (blue * 256.0f);
}

inline unsigned char Color::getAlpha8() const {
	if(alpha > 0.999f) return 255;
	if(alpha < 0.0f) return 0;
	return (unsigned char) alpha * 256;
}

inline void Color::setBlack() { red = green = blue = 0.0f; }
	
inline void Color::setWhite() { red = green = blue = 1.0f; }

inline void Color::setGrey(float shade) { red = green = blue = shade; }

inline Color Color::lerp(const Color& col2, float factor) const {
	return (*this) * factor + col2 * (1.0f - factor);
}

inline Color Color::operator+(const Color& col2) const {
	return Color(red + col2.getRed(), green + col2.getGreen(),
					 blue + col2.getBlue());
}

inline Color Color::operator-(const Color& col2) const {
	return Color(red - col2.getRed(), green - col2.getGreen(),
					 blue - col2.getBlue());
}

inline Color Color::operator*(const Color& col2) const {
	return Color(red * col2.getRed(), green * col2.getGreen(),
					 blue * col2.getBlue());
}

inline Color Color::operator*(float factor) const {
	return Color(red * factor, green * factor, blue * factor);
}

inline Color Color::operator/(float factor) const {
	return Color(red / factor, green / factor, blue / factor);
}

inline bool Color::operator==(const Color& col2) const {
	if(red != col2.getRed()) return false;
	if(green != col2.getGreen()) return false;
	if(blue != col2.getBlue()) return false;
	if(alpha != col2.getAlpha()) return false;
	return true;
}

inline bool Color::operator!=(const Color& col2) const {
	return !((*this) == col2);
}

inline Color Color::operator+=(const Color& col2) {
	(*this) = (*this) + col2;
	return (*this);
}

inline Color Color::operator-=(const Color& col2) {
	(*this) = (*this) - col2;
	return (*this);
}

inline Color Color::operator*=(const Color& col2) {
	(*this) = (*this) * col2;
	return (*this);
}

inline Color Color::operator*=(float factor) {
	(*this) = (*this) * factor;
	return (*this);
}

inline Color Color::operator/=(float factor) {
	(*this) = (*this) / factor;
	return (*this);
}

inline Color Color::operator+(void) const { return (*this); }

inline Color Color::operator-(void) const {
	return Color(-red, -green, -blue);
}

inline Color32::Color32() { }

inline Color32::Color32(Color col) {
	comp[0] = col.getRed8();
	comp[1] = col.getGreen8();
	comp[2] = col.getBlue8();
	comp[3] = col.getAlpha8();
}

inline Color32::Color32(unsigned char red, unsigned char green, unsigned char blue, unsigned char alpha) {
	comp[0] = red;
	comp[1] = green;
	comp[2] = blue;
	comp[3] = alpha;
}

inline Color32::Color32(unsigned char red, unsigned char green, unsigned char blue) {
	comp[0] = red;
	comp[1] = green;
	comp[2] = blue;
	comp[3] = 255;
}

inline Color32 Color32::operator=(Color col) {
	comp[0] = col.getRed8();
	comp[1] = col.getGreen8();
	comp[2] = col.getBlue8();
	comp[3] = col.getAlpha8();
	return (*this);
}

inline unsigned char& Color32::operator()(int index) {
	return comp[index];
}

inline unsigned char& Color32::refComp(int index) {
	return comp[index];
}

inline unsigned char* Color32::ptrComp(int index) {
	return comp + index;
}

inline int color32SSD(const Color32& col1, const Color32& col2) {
	int redDiff = col1.comp[0] - col2.comp[0];
	int greenDiff = col1.comp[1] - col2.comp[1];
	int blueDiff = col1.comp[2] - col2.comp[2];
	return redDiff * redDiff + greenDiff * greenDiff + blueDiff * blueDiff;
}

inline Color32 color32lerp(const Color32& col1, const Color32& col2, float factor) {
	unsigned char red = (unsigned char) ((factor * (float) col1.comp[0]) + (1.0f - factor) * ((int) col2.comp[0]));
	unsigned char green = (unsigned char) ((factor * (float) col1.comp[1]) + (1.0f - factor) * ((int) col2.comp[1]));
	unsigned char blue = (unsigned char) ((factor * (float) col1.comp[2]) + (1.0f - factor) * ((int) col2.comp[2]));
	unsigned char alpha = 255;
	return Color32(red, green, blue, alpha);
}

inline Color32 Color32::operator+(const Color32& col2) const {
	float red = (float) comp[0] / 256.0f;
	float green = (float) comp[1] / 256.0f;
	float blue = (float) comp[2] / 256.0f;
	float red2 = (float) col2.comp[0] / 256.0f;
	float green2 = (float) col2.comp[1] / 256.0f;
	float blue2 = (float) col2.comp[2] / 256.0f;
	return Color32(Color(red + red2, blue + blue2, green + green2));
}

inline Color32 Color32::operator*(float factor) const {
	float red = (float) comp[0] / 256.0f;
	float green = (float) comp[1] / 256.0f;
	float blue = (float) comp[2] / 256.0f;
	return Color32(Color(red, green, blue) * factor);
}

inline Color32 operator*(float factor, const Color32& col){
	return col * factor;
}

#endif
