
//#include  <GL/gl.h> 
//#include  <GL/glu.h>
//#include  <GL/glext.h>     
#include <windows.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <Gl/glut.h>
#include "RGBpixmap.h"

RGBpixmap pix;

 #ifdef _WIN32 
#define drand48() ((float)rand()/RAND_MAX)
 #endif


#define PI 3.14285714

/* #define SCREEN_SAVER_MODE */

#define PS_GRAVITY -9.8
#define PS_WATERFALL  1 
#define PS_FOUNTAIN   0


typedef struct {
    float x, y, z;
    float radius;
} PSsphere;

typedef struct {
    float position[3];			/* current position */
    float previous[3];			/* previous position */
    float velocity[3];			/* velocity (magnitude & direction) */
    float dampening;			/* % of energy lost on collision */
    int alive;				/* is this particle alive? */
} PSparticle;

typedef struct {
        float tea_x;
        float tea_y;
        float tea_z;
}PSteapot;

PSteapot teapot = {0.75,2.21,-0.05};

float bounce = 0.92, speedy = 0;
int bounce_gravity = 2;
bool rolling = false, bouncing = false;

PSparticle* particles = NULL;
#define NUM_SPHERES 1
PSsphere spheres[NUM_SPHERES] = {	/* position of spheres */
    { 0, 0.5, 0, 0.5 }
};
int num_particles = 1000;
int type = PS_WATERFALL;
float dis_x=0, dis_y=0, dis_z=0;
int points = 0;
int draw_spheres = 1;
int frame_rate = 1;
float frame_time = 0;
float flow = 500;
float slow_down = 1;

float spin_x = 0;
float spin_y = 0;
float spin_sphere = 0;
int point_size = 3;


/* timedelta: returns the number of seconds that have elapsed since
   the previous call to the function. */
#if defined(_WIN32)
#include <sys/timeb.h>
#else
#include <limits.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/times.h>
#endif
#ifndef CLK_TCK
#define CLK_TCK 1000
#endif
float
timedelta(void)
{
    static long begin = 0;
    static long finish, difference;

#if defined(_WIN32)
    static struct timeb tb;
    ftime(&tb);
    finish = tb.time*1000+tb.millitm;
#else
    static struct tms tb;
    finish = times(&tb);
#endif

    difference = finish - begin;
    begin = finish;

    return (float)difference/(float)CLK_TCK;
}



int
fequal(float a, float b)
{
    float epsilon = 0.1;
    float f = a - b;
    
    if (f < epsilon && f > -epsilon)
	return 1;
    else
	return 0;
}


void
psTimeStep(PSparticle* p, float dt)
{
    if (p->alive == 0)
	return;

    p->velocity[0] += 0;
    p->velocity[1] += PS_GRAVITY*dt;
    p->velocity[2] += 0;

    p->previous[0] = p->position[0];
    p->previous[1] = p->position[1];
    p->previous[2] = p->position[2];

    p->position[0] += p->velocity[0]*dt;
    p->position[1] += p->velocity[1]*dt;
    p->position[2] += p->velocity[2]*dt;
}




void
psNewParticle(PSparticle* p, float dt)
{
    if (type == PS_WATERFALL) {
	p->velocity[0] = -2*(drand48()-0.0);
	p->velocity[1] = 0;
	p->velocity[2] = 0.5*(drand48()-0.0);
	p->position[0] = 0.25+dis_x;
	p->position[1] = 2+dis_y;
	p->position[2] = -0.05+dis_z;
	p->previous[0] = p->position[0];
	p->previous[1] = p->position[1];
	p->previous[2] = p->position[2];
	p->dampening = 0.45*drand48();
	p->alive = 1;
    } else if (type == PS_FOUNTAIN) {
	p->velocity[0] = 2*(drand48()-0.5);
	p->velocity[1] = 5;
	p->velocity[2] = 2*(drand48()-0.5);
	p->position[0] = 0;
	p->position[1] = 1;
	p->position[2] = 0;
	p->previous[0] = p->position[0];
	p->previous[1] = p->position[1];
	p->previous[2] = p->position[2];
	p->dampening = 0.35*drand48();
	p->alive = 1;
    }

    psTimeStep(p, 2*dt*drand48());
}

void sphereBounce(void) {

	speedy = speedy + (float)bounce_gravity/100.0;
	
	spheres[0].y -= speedy/5 ;
	
	if (spheres[0].y < 0.5) {
		spheres[0].y = 0.5 ;
		speedy *= -bounce ;
	}
}
void sphereRoll() {
	spheres[0].x = -0.001;
	spin_sphere +=5;
	spheres[0].z -= 0.0437;
	if ( spheres[0].z < -2.5 ) {
		spheres[0].z = 0;
		spheres[0].x = 0;
		spin_sphere = 0;
		rolling = false;
	}
}

void
psBounce(PSparticle* p, float dt)
{
    float s;

    if (p->alive == 0)
	return;

    /* since we know it is the ground plane, we only need to
       calculate s for a single dimension. */
    s = -p->previous[1]/p->velocity[1];

    p->position[0] = (p->previous[0] + p->velocity[0] * s + 
		      p->velocity[0] * (dt-s) * p->dampening);
    p->position[1] = -p->velocity[1] * (dt-s) * p->dampening; /* reflect */
    p->position[2] = (p->previous[2] + p->velocity[2] * s + 
		      p->velocity[2] * (dt-s) * p->dampening);

    /* damp the reflected velocity (since the particle hit something,
       it lost some energy) */
    p->velocity[0] *=  p->dampening;
    p->velocity[1] *= -p->dampening;		/* reflect */
    p->velocity[2] *=  p->dampening;
}

void
psCollideSphere(PSparticle* p, PSsphere* s)
{
    float vx = p->position[0] - s->x;
    float vy = p->position[1] - s->y;
    float vz = p->position[2] - s->z;
    float distance;

    if (p->alive == 0)
	return;

    distance = sqrt(vx*vx + vy*vy + vz*vz);

    if (distance < s->radius) {
#if 0
	vx /= distance;  vy /= distance;  vz /= distance;
	d = 2*(-vx*p->velocity[0] + -vy*p->velocity[1] + -vz*p->velocity[2]);
	p->velocity[0] += vx*d*2;
	p->velocity[1] += vy*d*2;
	p->velocity[2] += vz*d*2;
	d = sqrt(p->velocity[0]*p->velocity[0] + 
		 p->velocity[1]*p->velocity[1] +
		 p->velocity[2]*p->velocity[2]);
	p->velocity[0] /= d;
	p->velocity[1] /= d;
	p->velocity[2] /= d;
#else
	p->position[0] = s->x+(vx/distance)*s->radius;
	p->position[1] = s->y+(vy/distance)*s->radius;
	p->position[2] = s->z+(vz/distance)*s->radius;
	p->previous[0] = p->position[0];
	p->previous[1] = p->position[1];
	p->previous[2] = p->position[2];
	p->velocity[0] = vx/distance;
	p->velocity[1] = vy/distance;
	p->velocity[2] = vz/distance;
#endif
    }
}


void
reshape(int width, int height)
{
    float black[] = { 0, 0, 0, 0 };

    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60, (float)width/height, 0.1, 1000);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    gluLookAt(0, 1, 3, 0, 1, 0, 0, 1, 0);
    glFogfv(GL_FOG_COLOR, black);
    glFogf(GL_FOG_START, 2.5);
    glFogf(GL_FOG_END, 4);
    glEnable(GL_FOG);
    glFogi(GL_FOG_MODE, GL_LINEAR);
    glPointSize(point_size);
    glEnable(GL_POINT_SMOOTH);
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_COLOR_MATERIAL);
    glEnable(GL_DEPTH_TEST);
    glEnable(GL_LIGHT0);
	glEnable(GL_DEPTH_TEST);

    timedelta();
}

GLuint name;
PSsphere e,p, cc;
void
display(void)
{
    static int i;
    static float c;
    static int j = 0;
    static char s[32];
    static int frames = 0;
    float num, oldvalue;

    glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
    glPushMatrix();

    glRotatef(spin_y, 1, 0, 0);
    glRotatef(spin_x, 0, 1, 0);

    glEnable(GL_LIGHTING);
   
   glPushMatrix();
	
    glTranslatef(teapot.tea_x,teapot.tea_y,teapot.tea_z);
    glColor3ub(210, 210, 210);   
    glRotatef(180,0,1,0);
    glRotatef(-40,0,0,1);
    glutSolidTeapot(0.3);

    glPopMatrix();
	
	glEnable(GL_TEXTURE_2D);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	pix.delCheckerboard();
	pix.makeCheckerboard();
	pix.deleteTexture();
	pix.setTexture();
	glBindTexture(GL_TEXTURE_2D, pix.getName());

    if (draw_spheres>0) {
	for (i = 0; i < draw_spheres; i++) {
	    glPushMatrix();    
		glRotatef(spin_sphere, spheres[i].x , spheres[i].y, spheres[i].z);
        glColor3ub(120,120,120);

		//adding texture to the sphere

	   int ii,j;
	   double theta1,theta2,theta3;
	   double r = spheres[i].radius;
	   int n = 180;
	   cc=spheres[i];


	   for (j=0;j<n/2;j++) {
		  theta1 = j * (2.0 * PI) / n - (PI / 2.0);
		  theta2 = (j + 1) * (2.0 * PI) / n - (PI / 2.0);

		  glBegin(GL_QUAD_STRIP);
		  for (ii=0;ii<=n;ii++) {
			 theta3 = ii * (PI * 2.0) /  n;

			 e.x = cos(theta2) * cos(theta3);
			 e.y = sin(theta2);
			 e.z = cos(theta2) * sin(theta3);
			 p.x = cc.x + r * e.x;
			 p.y = cc.y + r * e.y;
			 p.z = cc.z + r * e.z;

			 glNormal3f(e.x,e.y,e.z);
			 glTexCoord2f(ii/(double)n,2*(j+1)/(double)n);
			 glVertex3f(p.x,p.y,p.z);

			 e.x = cos(theta1) * cos(theta3);
			 e.y = sin(theta1);
			 e.z = cos(theta1) * sin(theta3);
			 p.x = cc.x + r * e.x;
			 p.y = cc.y + r * e.y;
			 p.z = cc.z + r * e.z;

			 glNormal3f(e.x,e.y,e.z);
			 glTexCoord2f(ii/(double)n,2*j/(double)n);
			 glVertex3f(p.x,p.y,p.z);
		  }
		  glEnd();
	   }

	//    glutSolidSphere(spheres[i].radius, 180, 180);
	    glPopMatrix();
	 }
    }
	glDisable(GL_TEXTURE_2D);
	//glDisable(GL_TEXTURE_GEN_S);
	//glDisable(GL_TEXTURE_GEN_T);
    glDisable(GL_LIGHTING);
	

	glEnable(GL_TEXTURE_2D);
	pix.deallocate();
	pix.readBMPFile("stone.bmp");
	pix.deleteTexture();
	pix.setupTexture();
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
	glBindTexture(GL_TEXTURE_2D, pix.getName());
	
	glBegin(GL_QUADS);
    glColor3ub(250 ,250 ,150 );
    glTexCoord2f(-3,-3);	glVertex3f(-3, 0, -3);
    glTexCoord2f(-3,3);		glVertex3f(-3, 0, 3);
	glTexCoord2f(3,3);		glVertex3f(3, 0, 3);
	glTexCoord2f(3,-3);		glVertex3f(3, 0, -3);
    glEnd();



    glBegin(GL_QUADS);
    glColor3ub(250 ,250 ,250 );
    glTexCoord2f(-3,-3);	glVertex3f(-3, -3, -3);
    glTexCoord2f(-3,3);		glVertex3f(-3, 3, -3);
    glTexCoord2f(3,3);		glVertex3f(3, 3, -3);
    glTexCoord2f(3,-3);		glVertex3f(3, -3, -3);
    glEnd();

    

    glBegin(GL_QUADS);
    glColor3ub(250 ,250 ,250 );
    glTexCoord2f(-3,-3);	glVertex3f(-3, -3, -3);
	glTexCoord2f(-3,3);		glVertex3f(-3, -3, 3);
    glTexCoord2f(3,3);		glVertex3f(-3, 3, 3);
    glTexCoord2f(3,-3);		glVertex3f(-3, 3, -3);
    glEnd();

    
    glBegin(GL_QUADS);
    glColor3ub(250 ,250 ,250 );
    glTexCoord2f(-3,-3);		glVertex3f(3, -3, -3);
    glTexCoord2f(-3,3);		glVertex3f(3, -3, 3);
    glTexCoord2f(3,3);		glVertex3f(3, 3, 3);
    glTexCoord2f(3,-3);		glVertex3f(3, 3, -3);
    glEnd();

	glDisable(GL_TEXTURE_2D);
    
	glBegin(GL_LINES);
	glColor3ub(0 ,0 ,0 );
	glVertex3f(1, 0, -3);
	glVertex3f(1, 0, 3);
	glEnd();

	glBegin(GL_LINES);
	glColor3ub(0 ,0 ,0 );
	glVertex3f(-1, 0, 3);
	glVertex3f(-1, 0, -3);
	glEnd();

	glBegin(GL_LINES);
	glColor3ub(0 ,0 ,0 );
	glVertex3f(3, 0, 1);
	glVertex3f(-3, 0, 1);
	glEnd();

	glBegin(GL_LINES);
	glColor3ub(0 ,0 ,0 );
	glVertex3f(3, 0, -1);
	glVertex3f(-3, 0, -1);
	glEnd();


    if (points) {
	glBegin(GL_POINTS);
	
	for (i = 0; i < num_particles; i++) {
	    if (particles[i].alive == 0)
		continue;
	    c = particles[i].position[1]/2.1*255;
	    glColor3ub(c, 128+c*0.5, 255);
	    glVertex3fv(particles[i].position);
	}
	glEnd();
    } 
    
    
    else {
	glBegin(GL_LINES);
	for (i = 0; i < num_particles; i++) {
	    if (particles[i].alive == 0)
		continue;
	    c = particles[i].previous[1]/2.1*255;
	    glColor4ub(c, 128+c*0.5, 255, 32);
	    glVertex3fv(particles[i].previous);
	    c = particles[i].position[1]/2.1*255;
	    glColor4ub(c, 128+c*0.5, 255, 196);
	    glVertex3fv(particles[i].position);
	}
	glEnd();
    }

    

    glPopMatrix();
    glutSwapBuffers();
    
    }

void
idle(void)
{
    static int i, j;
    static int living = 0;		/* index to end of live particles */
    static float dt;
    static float last = 0;

    dt = timedelta();
    frame_time += dt;

#if 1
    /* slow the simulation if we can't keep the frame rate up around
       10 fps */
    if (dt > 0.1) {
	slow_down = 1.0/(100*dt);
    } else if (dt < 0.1) {
	slow_down = 1;
    }
#endif

    dt *= slow_down;

    /* resurrect a few particles */
    for (i = 0; i < flow*dt; i++) {
	psNewParticle(&particles[living], dt);
	living++;
	if (living >= num_particles)
	    living = 0;
    }

    for (i = 0; i < num_particles; i++) {
	psTimeStep(&particles[i], dt);

	/* collision with sphere? */
	if (draw_spheres) {
	    for (j = 0; j < draw_spheres; j++) {
		psCollideSphere(&particles[i], &spheres[j]);
	    }
	}

	/* collision with ground? */
	if (particles[i].position[1] <= 0) {
	    psBounce(&particles[i], dt);
	}

	/* dead particle? */
	if (particles[i].position[1] < 0.1 && 
	    fequal(particles[i].velocity[1], 0)) {
	    particles[i].alive = 0;
	}
    }

	if( bouncing == true)
		sphereBounce();
	if( rolling == true)
		sphereRoll();
    glutPostRedisplay();
}

void
visible(int state)
{
    if (state == GLUT_VISIBLE)
	glutIdleFunc(idle);
    else
	glutIdleFunc(NULL);
}

void
bail(int code)
{
    free(particles);
    exit(code);
}

#ifdef SCREEN_SAVER_MODE
void
ss_keyboard(char key, int x, int y)
{
    bail(0);
}

void
ss_mouse(int button, int state, int x, int y)
{
    bail(0);
}

void
ss_passive(int x, int y)
{
    static int been_here = 0;

    /* for some reason, GLUT sends an initial passive motion callback
       when a window is initialized, so this would immediately
       terminate the program.  to get around this, see if we've been
       here before. (actually if we've been here twice.) */

    if (been_here > 1)
	bail(0);
    been_here++;
}

#else

void
keyboard(unsigned char key, int x, int y)
{
    static int fullscreen = 0;
    static int old_x = 50;
    static int old_y = 50;
    static int old_width = 512;
    static int old_height = 512;
    static int s = 0;
    float num;

    switch (key) {
    case 27:
        bail(0);
	break;

    case 's':
	draw_spheres++;
	if (draw_spheres > NUM_SPHERES)
	    draw_spheres = 0;
	break;

    case 'S':
	printf("PSsphere spheres[NUM_SPHERES] = {/* position of spheres */\n");
	for (s = 0; s < NUM_SPHERES; s++) {
	    printf("  { %g, %g, %g, %g },\n", 
		   spheres[s].x, spheres[s].y,
		   spheres[s].z, spheres[s].radius);
	}
	printf("};\n");
	break;

    case 'l':
	points = !points;
	break;

    case 'P':
	point_size++;
	glPointSize(point_size);
	break;
	
    case 'p':
	point_size--;
	if (point_size < 1)
	    point_size = 1;
	glPointSize(point_size);
	break;
	
    case '+':
	flow += 100;
	if (flow > num_particles)
	    flow = num_particles;
	printf("%g particles/second\n", flow);
	break;

    case '-':
	flow -= 100;
	if (flow < 0)
	    flow = 0;
	printf("%g particles/second\n", flow);
	break;

    case '#':
	frame_rate = !frame_rate;
	break;

    case '~':
	fullscreen = !fullscreen;
	if (fullscreen) {
	    old_x = glutGet(GLUT_WINDOW_X);
	    old_y = glutGet(GLUT_WINDOW_Y);
	    old_width = glutGet(GLUT_WINDOW_WIDTH);
	    old_height = glutGet(GLUT_WINDOW_HEIGHT);
	    glutFullScreen();
	} else {
	    glutReshapeWindow(old_width, old_height);
	    glutPositionWindow(old_x, old_y);
	}
	break;

    case '!':
	s++;
	if (s >= NUM_SPHERES)
	    s = 0;
	break;

    case '4':
	if(spheres[s].x - 0.05 >= -2.5)
		spheres[s].x -= 0.05;
	break;

    case '6':
	if(spheres[s].x + 0.05 <= 2.5)
		spheres[s].x += 0.05;
	break;

    case '2':
         if(((spheres[s].y - spheres[s].radius) - 0.05) > 0)
         	spheres[s].y -= 0.05;
	break;

    case '8':
         
         num = sqrt( pow((spheres[s].x - (teapot.tea_x)),2) 
           + pow((spheres[s].y - (teapot.tea_y)),2)
           + pow((spheres[s].z - (teapot.tea_z)),2) );
           
        if( num >= spheres[s].radius )
        {
            printf("NUM1 %f %f\n", num, spheres[s].radius);
             spheres[s].y += 0.05;
        }          
     break;

    case '7':
	if(spheres[s].z - 0.05 >= -2.5)
		spheres[s].z -= 0.05;
	break;

    case '3':
		spheres[s].z += 0.05;
	break;

    case '9':
    if(spheres[s].radius < 0.9 )
    {
    	spheres[s].radius += 0.05;
    	if(spheres[s].y <= spheres[s].radius)
    	{
              spheres[s].y = spheres[s].radius;
        }
        
        num = sqrt( pow((spheres[s].x - (teapot.tea_x)),2) 
                  + pow((spheres[s].y - (teapot.tea_y)),2)
                  + pow((spheres[s].z - (teapot.tea_z)),2) );
           
           printf("NUM %f %f\n", num, spheres[s].radius);
            printf("NUMMM %f %f %f\n", (spheres[s].x - (teapot.tea_x)),
                                       (spheres[s].y - (teapot.tea_y)),
                                       (spheres[s].z - (teapot.tea_z)));
        if( num + 0.05  >= spheres[s].radius )
        {
            
             spheres[s].radius += 0.05;
        }
    } 
 
	break;

    case '1':
		{
			spheres[s].radius -= 0.05;
			spheres[s].y-=0.05;
		}
	break;
	
	case 'z':
	num = sqrt( pow((spheres[0].x - (teapot.tea_x)),2) 
                + pow((spheres[0].y - (teapot.tea_y)),2)
                + pow((spheres[0].z - (teapot.tea_z)),2) );
	if (( num  >= spheres[0].radius + 0.3) && (teapot.tea_x + 0.05 <= 2.7)) {
		teapot.tea_x +=0.05;
		dis_x = dis_x +  0.05;
	}
    break;
    
    case 'x':
	teapot.tea_y +=0.05;
	dis_y = dis_y +  0.05;
	break;         
   	
    case 'c':
	num = sqrt( pow((spheres[0].x - (teapot.tea_x)),2) 
                + pow((spheres[0].y - (teapot.tea_y)),2)
                + pow((spheres[0].z - (teapot.tea_z)),2) );
	if ( num  >= spheres[0].radius + 0.3) {
		teapot.tea_z +=0.05;
		dis_z = dis_z +  0.05;
	}
    break;
    
    case 'y':
	num = sqrt( pow((spheres[0].x - (teapot.tea_x)),2) 
                + pow((spheres[0].y - (teapot.tea_y)),2)
                + pow((spheres[0].z - (teapot.tea_z)),2) );
	if (( num  >= spheres[0].radius + 0.3 + 0.2) && (teapot.tea_x - 0.05 >= -2.7)) {
		teapot.tea_x -= 0.05;
		dis_x = dis_x -  0.05;
	}
    break;
    
    case 'u':
	num = sqrt( pow((spheres[0].x - (teapot.tea_x)),2) 
               + pow((spheres[0].y - (teapot.tea_y)),2)
               + pow((spheres[0].z - (teapot.tea_z)),2) );
	if (( num  >= spheres[0].radius + 0.3)&& (teapot.tea_y - 0.05 >= 0.3)) {
		teapot.tea_y -= 0.05;
		dis_y = dis_y -  0.05;
	}
    break;
    
    case 'i':
	num = sqrt( pow((spheres[0].x - (teapot.tea_x)),2) 
               + pow((spheres[0].y - (teapot.tea_y)),2)
               + pow((spheres[0].z - (teapot.tea_z)),2) );
	if (( num  >= spheres[0].radius + 0.3) && (teapot.tea_z - 0.05 >= -2.7)) {
		teapot.tea_z -= 0.05;
		dis_z = dis_z -  0.05;
	}
    break;

	case 'b':
		spheres[0].y = 1.5;
		sphereBounce();
		bouncing = true;
		break;
	case 'n':
		spheres[0].y = 0.5;
		bouncing = false;
		break;
	case 'r':
		sphereRoll();
		rolling = true;
		break;
    }
}

void
menu(int item)
{
    keyboard((unsigned char)item, 0, 0);
}

void
menustate(int state)
{
    /* hook up a fake time delta to avoid jumping when menu comes up */
    if (state == GLUT_MENU_NOT_IN_USE)
	timedelta();
}
#endif

int old_x, old_y;

void
mouse(int button, int state, int x, int y)
{
    old_x = x;
    old_y = y;

    glutPostRedisplay();
}

void
motion(int x, int y)
{
    spin_x = x - old_x;
    spin_y = y - old_y;

    glutPostRedisplay();
}

int
main(int argc, char** argv)
{
    glutInitDisplayMode(GLUT_RGB|GLUT_DEPTH|GLUT_DOUBLE);
    glutInitWindowPosition(0, 0);
    glutInitWindowSize(640, 480);
    glutInit(&argc, argv);

    glutCreateWindow("Teapot - Fluid Flow Modelling");

    glutDisplayFunc(display);
    glutReshapeFunc(reshape);

#ifdef SCREEN_SAVER_MODE
    glutPassiveMotionFunc(ss_passive);
    glutKeyboardFunc(ss_keyboard);
    glutMouseFunc(ss_mouse);
    glutSetCursor(GLUT_CURSOR_NONE);
    glutFullScreen(); 
#else
    glutMotionFunc(motion);
    glutMouseFunc(mouse);
    glutKeyboardFunc(keyboard);
#endif

    glutMenuStateFunc(menustate);
    glutCreateMenu(menu);
    glutAddMenuEntry("Fluid Flow Modelling", 0);
    glutAddMenuEntry("", 0);
    glutAddMenuEntry("   Less flow --    [-]", '-');
    glutAddMenuEntry("   More flow --    [+]", '+');
    glutAddMenuEntry("   positive in x --  [z]", 'z');
    glutAddMenuEntry("   positive in y --  [x]", 'x');
    glutAddMenuEntry("   positive in z --  [c]", 'c');
    glutAddMenuEntry("   negative in x -- [y]", 'y');
    glutAddMenuEntry("   negative in y -- [u]", 'u');
    glutAddMenuEntry("   negative in z -- [i]", 'i');
	glutAddMenuEntry("   bounce the ball -- [b]", 'b');
	glutAddMenuEntry("   roll the ball down -z -- [r]", 'r');
    glutAddMenuEntry("", 0);
    glutAddMenuEntry("", 0);
    glutAddMenuEntry("[Esc] Quit", 27);
    glutAttachMenu(GLUT_RIGHT_BUTTON);

    particles = (PSparticle*)malloc(sizeof(PSparticle) * num_particles);

    glutVisibilityFunc(visible);
    glutMainLoop();
    return 0;
}
